@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 # Author: Natalia B. Bidart <natalia.bidart@canonical.com>
+# Author: Alejandro J. Cura <alecu@canonical.com>
 #
-# Copyright (C) 2009 Canonical Ltd.
+# Copyright (C) 2009, 2011 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License version 3,
@@ -20,16 +21,21 @@
 import StringIO
 import os
 import sys
+import time
 import unittest
 import uuid
 
-from twisted.internet.defer import Deferred
+from twisted.application import internet, service
+from twisted.internet import defer
+from twisted.internet.defer import Deferred, inlineCallbacks
 from twisted.trial.unittest import TestCase as TwistedTestCase
+from twisted.web import server, resource
 
 from ubuntuone.storageprotocol import protocol_pb2, sharersp, delta, request
 from ubuntuone.storageprotocol.client import (
     StorageClient, CreateUDF, ListVolumes, DeleteVolume, GetDelta, Unlink,
     Authenticate, MakeFile, MakeDir, PutContent, Move, BytesMessageProducer,
+    oauth, TwistedTimestampChecker, tx_timestamp_checker,
 )
 from ubuntuone.storageprotocol import volumes
 from tests import test_delta_info
@@ -464,8 +470,9 @@ class RequestTestCase(TwistedTestCase):
 
     request_class = request.Request
 
+    @defer.inlineCallbacks
     def setUp(self):
-        """Initialize testing protocol."""
+        yield super(RequestTestCase, self).setUp()
         import types
         self.request_class = types.ClassType(self.request_class.__name__,
                                              (self.request_class,), {})
@@ -476,21 +483,16 @@ class CreateUDFTestCase(RequestTestCase):
 
     request_class = CreateUDF
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize testing protocol."""
-        super(CreateUDFTestCase, self).setUp()
+        yield super(CreateUDFTestCase, self).setUp()
         self.protocol = FakedProtocol()
         self.request = self.request_class(self.protocol, path=PATH, name=NAME)
         self.request.error = faked_error
         self.done_called = False
         self.request.done = was_called(self, 'done_called')
 
-    def tearDown(self):
-        """Clean up."""
-        self.protocol = None
-        self.request = None
-        super(CreateUDFTestCase, self).tearDown()
-
     def test_init(self):
         """Test request creation."""
         self.assertEquals(PATH, self.request.path)
@@ -532,20 +534,16 @@ class ListVolumesTestCase(RequestTestCase):
 
     request_class = ListVolumes
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize testing protocol."""
-        super(ListVolumesTestCase, self).setUp()
+        yield super(ListVolumesTestCase, self).setUp()
         self.protocol = FakedProtocol()
         self.request = self.request_class(self.protocol)
         self.request.error = faked_error
         self.done_called = False
         self.request.done = was_called(self, 'done_called')
 
-    def tearDown(self):
-        """Clean up."""
-        self.protocol = None
-        self.request = None
-
     def test_init(self):
         """Test request creation."""
         self.assertEquals([], self.request.volumes)
@@ -618,20 +616,16 @@ class DeleteVolumeTestCase(RequestTestCase):
 
     request_class = DeleteVolume
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize testing protocol."""
-        super(DeleteVolumeTestCase, self).setUp()
+        yield super(DeleteVolumeTestCase, self).setUp()
         self.protocol = FakedProtocol()
         self.request = self.request_class(self.protocol, volume_id=VOLUME)
         self.request.error = faked_error
         self.done_called = False
         self.request.done = was_called(self, 'done_called')
 
-    def tearDown(self):
-        """Clean up."""
-        self.protocol = None
-        self.request = None
-
     def test_init(self):
         """Test request creation."""
         self.assertEquals(str(VOLUME), self.request.volume_id)
@@ -665,20 +659,16 @@ class GetDeltaTestCase(RequestTestCase):
 
     request_class = GetDelta
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize testing protocol."""
-        super(GetDeltaTestCase, self).setUp()
+        yield super(GetDeltaTestCase, self).setUp()
         self.protocol = FakedProtocol()
         self.request = self.request_class(self.protocol, SHARE, 0)
         self.request.error = faked_error
         self.done_called = False
         self.request.done = was_called(self, 'done_called')
 
-    def tearDown(self):
-        """Clean up."""
-        self.protocol = None
-        self.request = None
-
     def test_init(self):
         """Test request creation."""
         self.assertEquals(str(SHARE), self.request.share_id)
@@ -794,6 +784,142 @@ class TestAuth(RequestTestCase):
         self.assertEqual(len(msg.auth_parameters), 1)
         self.assertEqual(len(msg.metadata), 0)
 
+    def test_oauth_authenticate_uses_server_timestamp(self):
+        """The oauth authentication uses the server timestamp."""
+        fromcandt_call = []
+
+        fake_token = oauth.OAuthToken('token', 'token_secret')
+        fake_consumer = oauth.OAuthConsumer('consumer_key', 'consumer_secret')
+
+        fake_timestamp = object()
+        timestamp_d = Deferred()
+        self.patch(tx_timestamp_checker, "get_faithful_time",
+                   lambda: timestamp_d)
+        original_fromcandt = oauth.OAuthRequest.from_consumer_and_token
+
+        @staticmethod
+        def fake_from_consumer_and_token(**kwargs):
+            """A fake from_consumer_and_token."""
+            fromcandt_call.append(kwargs)
+            return original_fromcandt(**kwargs)
+
+        self.patch(oauth.OAuthRequest, "from_consumer_and_token",
+                   fake_from_consumer_and_token)
+        protocol = FakedProtocol()
+        protocol.oauth_authenticate(fake_consumer, fake_token)
+        self.assertEqual(len(fromcandt_call), 0)
+        timestamp_d.callback(fake_timestamp)
+        parameters = fromcandt_call[0]["parameters"]
+        self.assertEqual(parameters["oauth_timestamp"], fake_timestamp)
+
+
+class RootResource(resource.Resource):
+    """A root resource that logs the number of calls."""
+
+    isLeaf = True
+
+    def __init__(self, *args, **kwargs):
+        """Initialize this fake instance."""
+        self.count = 0
+        self.request_headers = []
+
+    def render_HEAD(self, request):
+        """Increase the counter on each render."""
+        self.request_headers.append(request.requestHeaders)
+        self.count += 1
+        return ""
+
+
+class MockWebServer(object):
+    """A mock webserver for testing."""
+
+    def __init__(self):
+        """Start up this instance."""
+        self.root = RootResource()
+        site = server.Site(self.root)
+        application = service.Application('web')
+        self.service_collection = service.IServiceCollection(application)
+        self.tcpserver = internet.TCPServer(0, site)
+        self.tcpserver.setServiceParent(self.service_collection)
+        self.service_collection.startService()
+
+    def get_url(self):
+        """Build the url for this mock server."""
+        port_num = self.tcpserver._port.getHost().port
+        return "http://localhost:%d/" % port_num
+
+    def stop(self):
+        """Shut it down."""
+        self.service_collection.stopService()
+
+
+class TimestampCheckerTestCase(TwistedTestCase):
+    """Tests for the timestamp checker."""
+
+    @inlineCallbacks
+    def setUp(self):
+        """Initialize a fake webserver."""
+        yield super(TimestampCheckerTestCase, self).setUp()
+        self.ws = MockWebServer()
+        self.addCleanup(self.ws.stop)
+        self.patch(TwistedTimestampChecker, "SERVER_URL", self.ws.get_url())
+
+    @inlineCallbacks
+    def test_returned_value_is_int(self):
+        """The returned value is an integer."""
+        checker = TwistedTimestampChecker()
+        t = yield checker.get_faithful_time()
+        self.assertEqual(type(t), int)
+
+    @inlineCallbacks
+    def test_first_call_does_head(self):
+        """The first call gets the clock from our web."""
+        checker = TwistedTimestampChecker()
+        yield checker.get_faithful_time()
+        self.assertEqual(self.ws.root.count, 1)
+
+    @inlineCallbacks
+    def test_second_call_is_cached(self):
+        """For the second call, the time is cached."""
+        checker = TwistedTimestampChecker()
+        yield checker.get_faithful_time()
+        yield checker.get_faithful_time()
+        self.assertEqual(self.ws.root.count, 1)
+
+    @inlineCallbacks
+    def test_after_timeout_cache_expires(self):
+        """After some time, the cache expires."""
+        fake_timestamp = 1
+        self.patch(time, "time", lambda: fake_timestamp)
+        checker = TwistedTimestampChecker()
+        yield checker.get_faithful_time()
+        fake_timestamp += TwistedTimestampChecker.CHECKING_INTERVAL
+        yield checker.get_faithful_time()
+        self.assertEqual(self.ws.root.count, 2)
+
+    @inlineCallbacks
+    def test_server_error_means_skew_not_updated(self):
+        """When server can't be reached, the skew is not updated."""
+        fake_timestamp = 1
+        self.patch(time, "time", lambda: fake_timestamp)
+        checker = TwistedTimestampChecker()
+        failing_get_server_time = lambda: defer.fail(FakedError())
+        self.patch(checker, "get_server_time", failing_get_server_time)
+        yield checker.get_faithful_time()
+        self.assertEqual(checker.skew, 0)
+        self.assertEqual(checker.next_check,
+                    fake_timestamp + TwistedTimestampChecker.ERROR_INTERVAL)
+
+    @inlineCallbacks
+    def test_server_date_sends_nocache_headers(self):
+        """Getting the server date sends the no-cache headers."""
+        checker = TwistedTimestampChecker()
+        yield checker.get_server_date_header(self.ws.get_url())
+        assert len(self.ws.root.request_headers) == 1
+        headers = self.ws.root.request_headers[0]
+        result = headers.getRawHeaders("Cache-Control")
+        self.assertEqual(result, ["no-cache"])
+
 
 class TestGenerationInRequests(RequestTestCase):
     """Base class for testing that actions that change the volume will
