@@ -19,7 +19,9 @@
 
 import uuid
 
+from twisted.internet import defer
 from twisted.trial.unittest import TestCase
+
 from ubuntuone.storageprotocol.sharersp import ShareResponse
 from ubuntuone.storageprotocol import protocol_pb2
 
@@ -88,8 +90,9 @@ class ShareResponseFromToMsgTest(TestCase):
 
     access_level = 'View'
 
+    @defer.inlineCallbacks
     def setUp(self):
-        """Setup."""
+        yield super(ShareResponseFromToMsgTest, self).setUp()
         self.msg = protocol_pb2.Message()
         self.msg.type = protocol_pb2.Message.SHARES_INFO
 
