@@ -20,7 +20,7 @@
 
 from __future__ import with_statement
 
-from twisted.internet import task
+from twisted.internet import defer, task
 from twisted.trial.unittest import TestCase as TwistedTestCase
 
 from ubuntuone.storageprotocol import client
@@ -52,18 +52,14 @@ class FakeClient(object):
 class BaseThrottlingTestCase(TwistedTestCase):
     """Base test case for ThrottlingStorageClientFactory."""
 
+    @defer.inlineCallbacks
     def setUp(self):
+        yield super(BaseThrottlingTestCase, self).setUp()
         self.client = FakeClient()
         self.factories = []
         self.clock = task.Clock()
-        # use our custom clock instead of the reactor for the callLater
-        self.old_callLater = client.ThrottlingStorageClientFactory.callLater
-        client.ThrottlingStorageClientFactory.callLater = self.clock.callLater
-
-    def tearDown(self):
-        for f in self.factories:
-            f.unregisterProtocol(None)
-        client.ThrottlingStorageClientFactory.callLater = self.old_callLater
+        self.patch(client.ThrottlingStorageClientFactory, 'callLater',
+                   self.clock.callLater)
 
     def create_factory(self, enabled, read_limit, write_limit):
         """Create a ThrottlingStorageClientFactory with the specified args."""
@@ -71,6 +67,7 @@ class BaseThrottlingTestCase(TwistedTestCase):
                                                      write_limit)
         tscf.client = self.client
         self.factories.append(tscf)
+        self.addCleanup(self.destroy_factory, tscf)
         return tscf
 
     def destroy_factory(self, factory):
@@ -82,8 +79,9 @@ class BaseThrottlingTestCase(TwistedTestCase):
 class TestProducingState(BaseThrottlingTestCase):
     """Tests for 'producing' state with different limit values."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        BaseThrottlingTestCase.setUp(self)
+        yield super(TestProducingState, self).setUp()
         self.tscf = self.create_factory(True, 3, 3)
 
     def test_under_write_limit(self):
