@@ -4,6 +4,7 @@
 # Author: Natalia B. Bidart <natalia.bidart@canonical.com>
 # Author: Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
 # Author: Facundo Batista <facundo@canonical.com>
+# Author: Alejandro J. Cura <alecu@canonical.com>
 #
 # Copyright 2009-2011 Canonical Ltd.
 #
@@ -30,13 +31,36 @@ from oauth import oauth
 from twisted.internet.protocol import ClientFactory
 from twisted.internet import reactor, defer
 from twisted.python import log
+from twisted.web import client
+from twisted.web.http_headers import Headers
 
 from ubuntuone.storageprotocol import (
-    protocol_pb2, request, sharersp, volumes, delta)
+    delta,
+    protocol_pb2,
+    request,
+    sharersp,
+    utils,
+    volumes,
+)
 
 log_debug = partial(log.msg, loglevel=logging.DEBUG)
 
 
+class TwistedTimestampChecker(utils.BaseTimestampChecker):
+    """A timestamp that's regularly checked with a server."""
+
+    @defer.inlineCallbacks
+    def get_server_date_header(self, server_url):
+        """Get the server date using twisted webclient."""
+        agent = client.Agent(reactor)
+        headers = Headers({'Cache-Control': ['no-cache']})
+        response = yield agent.request("HEAD", server_url, headers)
+        defer.returnValue(response.headers.getRawHeaders("Date")[0])
+
+
+tx_timestamp_checker = TwistedTimestampChecker()
+
+
 class StorageClient(request.RequestHandler):
     """A Basic Storage Protocol client."""
 
@@ -102,6 +126,7 @@ class StorageClient(request.RequestHandler):
         p.start()
         return p.deferred
 
+    @defer.inlineCallbacks
     def oauth_authenticate(self, consumer, token, metadata=None):
         """Authenticate to a server using the OAuth provider.
 
@@ -115,9 +140,11 @@ class StorageClient(request.RequestHandler):
         object when completed.
 
         """
+        timestamp = yield tx_timestamp_checker.get_faithful_time()
         req = oauth.OAuthRequest.from_consumer_and_token(
             oauth_consumer=consumer,
             token=token,
+            parameters={"oauth_timestamp": timestamp},
             http_method="GET",
             http_url="storage://server")
         req.sign_request(
@@ -128,7 +155,8 @@ class StorageClient(request.RequestHandler):
             (key, str(value)) for key, value in req.parameters.iteritems())
         p = Authenticate(self, auth_parameters, metadata=metadata)
         p.start()
-        return p.deferred
+        result = yield p.deferred
+        defer.returnValue(result)
 
     def handle_ROOT(self, message):
         """Handle incoming ROOT message.
